<BLOCK>
<LICENSE>
/*  Capsim (r) Text Mode Kernel (TMK) Block Library (Blocks)
 *  Copyright (C) 2002-2017  Silicon DSP Corporation
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    http://www.silicondsp.com
 *    Silicon DSP  Corporation
 *    Las Vegas, Nevada
 */

</LICENSE>

<BLOCK_NAME>
rsdecoder
</BLOCK_NAME> 

<COMMENTS>
<![CDATA[ 

/*
 * This is Capsim Block Implementation and Setup. See the subroutine eras_dec_rs.cpp for the rs decoder which is 
 * derived from the program "new_rs_erasures.c" by 
 * Robert Morelos-Zaragoza (robert at spectra.eng.hawaii.edu) and Hari Thirumoorthy
 * (harit at spectra.eng.hawaii.edu), Aug 1995
 *
 * Including Authors copyright notice below.
 * 
 * File:     general_RS.c
 * Title:    Encoder/decoder for RS codes in C
 * Authors:  Robert Morelos-Zaragoza (robert@spectra.eng.hawaii.edu)
 *           and Hari Thirumoorthy (harit@spectra.eng.hawaii.edu)
 * Date:     Aug 1995
 *
 *
 * ===============  Encoder/Decoder for RS codes in C =================
 *
 *
 * The encoding and decoding methods used in this program are based on the
 * book "Error Control Coding: Fundamentals and Applications", by Lin and
 * Costello, Prentice Hall, 1983. Most recently on "Theory and Practice of 
 * Error Control Codes", by R.E. Blahut.
 *
 *
 * NOTE:    
 *          The authors are not responsible for any malfunctioning of
 *          this program, nor for any damage caused by it. Please include the
 *          original program along with these comments in any redistribution.
 *
 * Portions of this program are from a Reed-Solomon encoder/decoder
 * in C, written by Simon Rockliff (simon@augean.ua.oz.au) on 21/9/89.
 *
 * COPYRIGHT NOTICE: This computer program is free for non-commercial purposes.
 * You may implement this program for any non-commercial application. You may 
 * also implement this program for commercial purposes, provided that you
 * obtain my written permission. Any modification of this program is covered
 * by this copyright.
 *
 * Copyright (c) 1995.  Robert Morelos-Zaragoza and Hari Thirumoorthy.
 *                      All rights reserved.
 *
 */
 

/*
 * 		Reed Solomon Decoding  decoder_rs()
 *
 *
 *
 * Performs ERRORS+ERASURES decoding of RS codes. If decoding is successful,
 * writes the codeword into data[] itself. Otherwise data[] is unaltered.
 *
 * Return number of symbols corrected, or -1 if codeword is illegal
 * or uncorrectable.
 * 
 * First "no_eras" erasures are declared by the calling program. Then, the
 * maximum # of errors correctable is t_after_eras = floor((NN-KK-no_eras)/2).
 * If the number of channel errors is not greater than "t_after_eras" the
 * transmitted codeword will be recovered. Details of algorithm can be found
 * in Richard Blahut's "Theory  and Practice  of Error Control  Codes", 1983 Addison Wesley Publishing
 *
 * 
 *    
 * Programmer:     Bin Huang
 * Date:           12/27/2002
 * Modified:       12/30/2002,standardize the output data type to 'int'
 *
 */
]]>
</COMMENTS> 

<INCLUDES>
<![CDATA[ 

#include "rs.h"
#include "reedSolomon.h"

]]>
</INCLUDES> 

<DECLARATIONS> 

	int i,j;
	int x;
	int no_samples;

</DECLARATIONS> 




<STATES>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> sampleCount </NAME>
		<VALUE> 0 </VALUE>
	</STATE>
	<STATE>
		<TYPE> dtype* </TYPE>
		<NAME> dat </NAME>
	</STATE>
	<STATE>
		<TYPE> int* </TYPE>
		<NAME> bina </NAME>
	</STATE>
	<STATE>
		<TYPE> int* </TYPE>
		<NAME> eras_po </NAME>
	</STATE>
	<STATE>
		<TYPE> gf* </TYPE>
		<NAME> Index_of </NAME>
	</STATE>
	<STATE>
		<TYPE> gf* </TYPE>
		<NAME> Alpha_to </NAME>
	</STATE>
	<STATE>
		<TYPE> gf* </TYPE>
		<NAME> Gg </NAME>
	</STATE>
	<STATE>
		<TYPE> int* </TYPE>
		<NAME> Pp </NAME>
	</STATE>
</STATES>



<PARAMETERS>
	<PARAM>
		<DEF> Total number of erasure </DEF>
		<TYPE> int </TYPE>
		<NAME> no_eras </NAME>
		<VALUE> 0 </VALUE>
	</PARAM>
</PARAMETERS>



<INPUT_BUFFERS>
	<BUFFER>
		<TYPE> int </TYPE>
		<NAME> data </NAME>
	</BUFFER>
</INPUT_BUFFERS>



<OUTPUT_BUFFERS>
	<BUFFER>
		<TYPE> int </TYPE>
		<NAME> dout </NAME>
	</BUFFER>
</OUTPUT_BUFFERS>

<INIT_CODE>
<![CDATA[ 

  /*
   * set the output buffer cell size to size of int
   */	
   
   /* 
    * Allocate Space
    */
if ((dat = (dtype*)calloc(NN,sizeof(dtype))) == NULL)
  {
	fprintf(stderr,"reed solomon decoder: can't allocate work space for dat\n");
	return(4);
  }
if ((eras_po = (int*)calloc(NN-KK,sizeof(int))) == NULL)
  {
	fprintf(stderr,"reed solomon decoder: can't allocate work space for erase_po\n");
	return(4);
  }
if ((Index_of = (gf*)calloc(NN+1,sizeof(gf))) == NULL)
  {
	fprintf(stderr,"reed solomon encoder: can't allocate work space for Index_of\n");
	return(4);
  }
if ((Alpha_to = (gf*)calloc(NN+1,sizeof(gf))) == NULL)
  {
	fprintf(stderr,"reed solomon encoder: can't allocate work space for  Alpha_to\n");
	return(4);
  }
 if ((Gg = (gf*)calloc(NN-KK+1,sizeof(gf))) == NULL)
  {
	fprintf(stderr,"reed solomon encoder: can't allocate work space for Gg\n");
	return(4);
  }
 if ((Pp = (int*)calloc(MM+1,sizeof(int))) == NULL)
  {
	fprintf(stderr,"reed solomon encoder: can't allocate work space for Pp\n");
	return(4);
  }
if ((bina = (int*)calloc(KK*MM,sizeof(int))) == NULL)
  {
	fprintf(stderr,"reed solomon encoder: can't allocate work space for bina\n");
	return(4);
  }
// (MM == 8)
/* 1+x^2+x^3+x^4+x^8 */
//             0  1  2  3  4  5  6  7  8
//Pp[MM+1] = { 1, 0, 1, 1, 1, 0, 0, 0, 1 };
  for(i=0;i<MM+1;i++);
    Pp[i] = 0;
  for(i=0;i<MM+1;i++)
   {
     switch(i) {
	  case 0:
	  case 2:
	  case 3:
	  case 4:
	  case 8:
	    Pp[i] = 1;
		break;
     }
   }
  SET_CELL_SIZE_OUT(0,sizeof(int));

]]>
</INIT_CODE> 


<MAIN_CODE>
<![CDATA[ 


   for (no_samples = MIN_AVAIL();no_samples>0; --no_samples)
     {

        IT_IN(0);        /*read input from buffer*/
        x = data(0);
		dat[sampleCount] = (dtype) x;
		sampleCount++;

       if(sampleCount == NN)
	    {
		  sampleCount = 0;

		  for(i=0;i<NN-KK;i++)
		   eras_po[i] =0;

		  // initialization: call init_rs.c

          init_rs(Alpha_to,Index_of,Gg,Pp);

		  // reed solomon decoder: call eras_dec_rs.c

		   eras_dec_rs(dat,eras_po,no_eras,Index_of,Alpha_to);

          // output the decoded data:

		  for(i=0;i<KK;i++)
		   {
		     for(j=0;j<MM;j++)
			   bina[j+i*MM] = ((int) dat[i] >> j) & 1;

           }

         for(i=0;i<KK*MM;i++)      // will be KK
		  {
		    
           if(IT_OUT(0)){
	            KrnOverflow("reed solomon decoder",0);
	            return(99);
	          }

           dout(0) = bina[i];


	      }

        }    // end if
			
			
	}       // end for

]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 


 free((char*)Index_of);
 free((char*)Alpha_to);
 free((char*)Gg);
 free((char*)Pp);
 free((char*)dat);
 free(eras_po);
 free(bina);


]]>
</WRAPUP_CODE> 



</BLOCK> 

