<BLOCK>
<LICENSE>
/*  Capsim (r) Text Mode Kernel (TMK) Block Library (Blocks)
 *  Copyright (C) 2002-2017  Silicon DSP Corporation
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License as published by the Free Software Foundation; either
 *    version 2.1 of the License, or (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *    http://www.silicondsp.com
 *    Silicon DSP  Corporation
 *    Las Vegas, Nevada
 */

</LICENSE>

<BLOCK_NAME>
rsencoder
</BLOCK_NAME> 

<COMMENTS>
<![CDATA[ 

/*
 * This file is derived from the program "new_rs_erasures.c" by Robert
 * Morelos-Zaragoza (robert at spectra.eng.hawaii.edu) and Hari Thirumoorthy
 * (harit at spectra.eng.hawaii.edu), Aug 1995
 *
 * Including Authors copyright notice below.
 * 
 * File:     general_RS.c
 * Title:    Encoder/decoder for RS codes in C
 * Authors:  Robert Morelos-Zaragoza (robert@spectra.eng.hawaii.edu)
 *           and Hari Thirumoorthy (harit@spectra.eng.hawaii.edu)
 * Date:     Aug 1995
 *
 *
 * ===============  Encoder/Decoder for RS codes in C =================
 *
 *
 * The encoding and decoding methods used in this program are based on the
 * book "Error Control Coding: Fundamentals and Applications", by Lin and
 * Costello, Prentice Hall, 1983. Most recently on "Theory and Practice of 
 * Error Control Codes", by R.E. Blahut.
 *
 *
 * NOTE:    
 *          The authors are not responsible for any malfunctioning of
 *          this program, nor for any damage caused by it. Please include the
 *          original program along with these comments in any redistribution.
 *
 * Portions of this program are from a Reed-Solomon encoder/decoder
 * in C, written by Simon Rockliff (simon@augean.ua.oz.au) on 21/9/89.
 *
 * COPYRIGHT NOTICE: This computer program is free for non-commercial purposes.
 * You may implement this program for any non-commercial application. You may 
 * also implement this program for commercial purposes, provided that you
 * obtain my written permission. Any modification of this program is covered
 * by this copyright.
 *
 * Copyright (c) 1995.  Robert Morelos-Zaragoza and Hari Thirumoorthy.
 *                      All rights reserved.
 *
 */
 

/*
 * 		rsencoder.s()
 *
 *    Reed-Solomon encoding:
 *
 * This file is derived from the program "new_rs_erasures.c" by Robert
 * Morelos-Zaragoza (robert at spectra.eng.hawaii.edu) and Hari Thirumoorthy
 * (harit at spectra.eng.hawaii.edu), Aug 1995
 * 
 * This is the Capim Block Implementation.
 * 
 * This code supports a symbol size from 2 bits up to 16 bits,
 * implying a block size of 3 2-bit symbols (6 bits) up to 65535
 * 16-bit symbols (1,048,560 bits). The code parameters are set in reedSolomon.h.
 *
 *    
 * In order to comply with the IEEE 802.16 wireless MAN standard, the symbol size should
 * be 8 bits(unsigned char). a symbol of (N,K,T) should be(255,239,8) where N:= the
 * number of bytes of ouput encoder, K := the number of bytes of input, T := the 
 * minimum number of bytes of corrected synbols.
 *
 * The input to encoder_rs.s is binary bits, they are first converted into
 * bytes, then apply to encode_rs.c.
 *
 *
 * Programmer:    Bin Huang
 * Date:          12/26/2002
 * Modified:      12/30/2002, add binary-to-bytes convertion
 *
 */
]]>
</COMMENTS> 

<INCLUDES>
<![CDATA[ 

#include "rs.h"
#include "reedSolomon.h"

]]>
</INCLUDES> 

<DECLARATIONS> 

	int i;
	int j;
	int x;
	int xx;
	int no_samples;

</DECLARATIONS> 




<STATES>
	<STATE>
		<TYPE> int </TYPE>
		<NAME> sampleCount </NAME>
		<VALUE> 0 </VALUE>
	</STATE>
	<STATE>
		<TYPE> int* </TYPE>
		<NAME> indat </NAME>
	</STATE>
	<STATE>
		<TYPE> dtype* </TYPE>
		<NAME> dat </NAME>
	</STATE>
	<STATE>
		<TYPE> dtype* </TYPE>
		<NAME> bbout </NAME>
	</STATE>
	<STATE>
		<TYPE> gf* </TYPE>
		<NAME> Index_of </NAME>
	</STATE>
	<STATE>
		<TYPE> gf* </TYPE>
		<NAME> Alpha_to </NAME>
	</STATE>
	<STATE>
		<TYPE> gf* </TYPE>
		<NAME> Gg </NAME>
	</STATE>
	<STATE>
		<TYPE> int* </TYPE>
		<NAME> Pp </NAME>
	</STATE>
</STATES>



<INPUT_BUFFERS>
	<BUFFER>
		<TYPE> int </TYPE>
		<NAME> data </NAME>
	</BUFFER>
</INPUT_BUFFERS>



<OUTPUT_BUFFERS>
	<BUFFER>
		<TYPE> int </TYPE>
		<NAME> bout </NAME>
	</BUFFER>
</OUTPUT_BUFFERS>

<INIT_CODE>
<![CDATA[ 

 
   
   /*
    * Allocate Space
    */
 if ((indat = (int*)calloc(KK*MM,sizeof(int))) == NULL)
  {
	fprintf(stderr,"reed solomon encoder: can't allocate work space for indat\n");
	return(4);
  }
 if((dat = (dtype*)calloc(KK,sizeof(dtype))) == NULL ||
   (bbout = (dtype*)calloc(NN-KK,sizeof(dtype))) == NULL) {
   fprintf(stderr,"Reed Solomon: can't allocate work space for dat or bbout \n");
   return(4);
   }
 if ((Index_of = (gf*)calloc(NN+1,sizeof(gf))) == NULL)
  {
	fprintf(stderr,"reed solomon encoder: can't allocate work space for Index_of \n");
	return(4);
  }
 if ((Alpha_to = (gf*)calloc(NN+1,sizeof(gf))) == NULL)
  {
	fprintf(stderr,"reed solomon encoder: can't allocate work space Alpha_to \n");
	return(4);
  }
 if ((Gg = (gf*)calloc(NN-KK+1,sizeof(gf))) == NULL)
  {
	fprintf(stderr,"reed solomon encoder: can't allocate work space for Gg\n");
	return(4);
  }
 if ((Pp = (int*)calloc(MM+1,sizeof(int))) == NULL)
  {
	fprintf(stderr,"reed solomon encoder: can't allocate work space for Pp\n");
	return(4);
  }
// (MM == 8)
/* 1+x^2+x^3+x^4+x^8 */
//             0  1  2  3  4  5  6  7  8
//Pp[MM+1] = { 1, 0, 1, 1, 1, 0, 0, 0, 1 };
  for(i=0;i<MM+1;i++);
    Pp[i] = 0;
  for(i=0;i<MM+1;i++)
   {
     switch(i) {
	  case 0:
	  case 2:
	  case 3:
	  case 4:
	  case 8:
	    Pp[i] = 1;
		break;
     }
    }
    
  /*
   * set the output buffer cell size to size of int
   */	
  SET_CELL_SIZE_OUT(0,sizeof(int));

]]>
</INIT_CODE> 


<MAIN_CODE>
<![CDATA[ 


   for (no_samples = MIN_AVAIL();no_samples>0; --no_samples)
    {
        IT_IN(0);
        x = data(0);
		indat[sampleCount] = x;
		sampleCount++;


		if(sampleCount == KK*MM)
		 {

		   sampleCount = 0;

		   //convert binary bits into bytes first:

            
		    for(i=0;i<KK;i++)
			 {
			  xx=0;
			  for(j=0;j<MM;j++)
			     xx +=indat[j+i*MM]<<j;

             dat[i] = (dtype) xx;
	//		 fprintf(stderr,"reed solomon:%d %d\n",i,dat[i]); 

             }


		   // initialization: call init_rs.c

		   init_rs(Alpha_to,Index_of,Gg,Pp);

   //        for(i=0;i<NN+1;i++) {
//		     fprintf(stderr,"init:%d %d\n",i,(int) Alpha_to[i]);  }

		   // reed solomon encoder: call the encode_rs.c

		   encode_rs(dat,bbout,Index_of,Alpha_to,Gg);
  
	//	   for(i=0;i<NN+1;i++) {
	//   	       fprintf(stderr,"after init:%d %d\n",i,(int) Alpha_to[i]); 
    //       }

		   // output the encoded bytes:

		   for(i=0;i<KK;i++)
		    {

			  if(IT_OUT(0)){
	            KrnOverflow("reed solomon encoder",0);
	            return(99);
	          }

             bout(0) = (int) dat[i];
		    }

       // append the redundancy to the output:

	     for(i=0;i<NN-KK;i++)
		  {
		    if(IT_OUT(0)){
	            KrnOverflow("reed solomon encoder",0);
	            return(99);
	          }

             bout(0) = (int) bbout[i];
          }


      }     // end if			
			
}


]]>
</MAIN_CODE> 

<WRAPUP_CODE>
<![CDATA[ 


 free((char*)Index_of);
 free((char*)Alpha_to);
 free((char*)Gg);
 free((char*)Pp);
 free((char*)dat);
 free((char*)bbout);
 free((char*)indat);

]]>
</WRAPUP_CODE> 



</BLOCK> 

